/*
 * "Hello World" example.
 *
 * This example prints 'Hello from Nios II' to the STDOUT stream. It runs on
 * the Nios II 'standard', 'full_featured', 'fast', and 'low_cost' example
 * designs. It runs with or without the MicroC/OS-II RTOS and requires a STDOUT
 * device in your system's hardware.
 * The memory footprint of this hosted application is ~69 kbytes by default
 * using the standard reference design.
 *
 * For a reduced footprint version of this template, and an explanation of how
 * to reduce the memory footprint for a given application, see the
 * "small_hello_world" template.
 *
 */

/*
 * "Hello World" example.
 *
 * This example prints 'Hello from Nios II' to the STDOUT stream. It runs on
 * the Nios II 'standard', 'full_featured', 'fast', and 'low_cost' example
 * designs. It runs with or without the MicroC/OS-II RTOS and requires a STDOUT
 * device in your system's hardware.
 * The memory footprint of this hosted application is ~69 kbytes by default
 * using the standard reference design.
 *
 * For a reduced footprint version of this template, and an explanation of how
 * to reduce the memory footprint for a given application, see the
 * "small_hello_world" template.
 *
 */


#include <stdio.h>
#include "CUart.h"
#include "terasic_includes.h"

const int MaxList = 9;


bool CommandRead(CUart   &Uart, char *pBuffer, int nBufferSize, int *prx_lenmand){
	int RxLen;
	char Data;
	bool bDone = false;
	int Len = 0;
	alt_u32 timeout;

	if (!Uart.Read((char *)&Data, sizeof(Data), &RxLen) || RxLen == 0)
			return false;

	*(pBuffer + 0) = Data;

	// read until find '\r'
	//timeout = alt_nticks() + alt_ticks_per_second()/1000;
	timeout = alt_nticks() + (alt_ticks_per_second()*30);
	Len = 1;
	while(!bDone){
		if (Uart.NoneBlockingRead((char *)&Data, sizeof(Data), &RxLen) && RxLen == sizeof(Data)){
			*(pBuffer + Len) = Data;
			Len++;
			if (Data == '\n')
				bDone = true;
		}else if (Len >= nBufferSize){
			// buffer full
			bDone = true;
		}else if (alt_nticks() > timeout){
			// timeout
			bDone = true;
		}else{
		}
		usleep(10);

	}
	*prx_lenmand = Len;
	return true;
}

bool SetupSend(CUart Uart,char *cmd)
{
	bool bFindOK=false;
	int  nReadLen;
    char szData[100];
    alt_u32 timeout;
    printf("TX-->%s",cmd);

    timeout = alt_nticks() + (alt_ticks_per_second()*30);

	Uart.WriteString(cmd);
	while(!bFindOK){
	  if(CommandRead(Uart, szData, sizeof(szData), &nReadLen)){
	      szData[nReadLen] = 0;
	     // printf("RX<--%s \n",szData);
	  }
	  if(!(strcmp(szData, "OK\r\n")))
		  bFindOK = true;
	  if(alt_nticks() > timeout){
		  printf("Time Out\n");
		  return false;
	  }

	}
	usleep(100000);
	return bFindOK;
}

bool GetDeviceList(CUart Uart, char *cmd){

	bool bFindOK=false;
	int  nReadLen;
    char szData[100];
    alt_u8 count=0;
    alt_u32 timeout;
    timeout = alt_nticks() + (alt_ticks_per_second()*30);
    printf("TX-->%s",cmd);
	Uart.WriteString(cmd);
	while(!bFindOK){
	  if(CommandRead(Uart, szData, sizeof(szData), &nReadLen)){
	      szData[nReadLen] = 0;
		  if(!(strcmp(szData, "OK\r\n"))){
			  bFindOK = true;
			  //printf("RX<--%s\n",szData);
		  }
		  else{
	        printf("Device[%d] : %s\n",count,szData);
           // strcpy(szDeviceData[count],szData);
            count++;
		  }
	  }

	  if(alt_nticks() > timeout){
		  printf("Time Out\n");
		  return false;
	  }
	}

	return bFindOK;

}


bool Connect_Device(CUart Uart, char *name, char **szDeviceData){

	bool bFindOK= false;
	bool bFindADDR= false;
	int  nReadLen;
    char szData[200];
    char szDevice_Addr[50][1];
    char SlaveAddrData[35];
    char * pch;
    //char str[] = "ESP32_SPP_Slave";
    alt_u8 count=0;
    alt_u32 timeout;
    timeout = alt_nticks() + (alt_ticks_per_second()*30);

	Uart.WriteString("AT+BTSTARTDISC=0,3,10\r\n");

	while(!bFindOK){
	  if(CommandRead(Uart, szData, sizeof(szData), &nReadLen)){
	      szData[nReadLen] = 0;
		  if(!(strcmp(szData, "OK\r\n"))){
			  bFindOK = true;
			  //printf("RX<--%s\n",szData);
		  }
		  else if(count < MaxList){
			strcpy(szDeviceData[count],szData);
			if(!bFindADDR){
				pch = strstr(szDeviceData[count],name);
	        	if(pch != NULL){
	        		sscanf(szDeviceData[count],"+BTSTARTDISC:\"%[^\"]",szDevice_Addr[0]);
	        		bFindADDR = true;
	        	}
			}
            count++;
		  }
		  if(alt_nticks() > timeout){
		  	  printf("Time Out\n");
		  	  return false;
	  	  }
	  }
	}

	if(bFindADDR){
		sprintf(SlaveAddrData,"AT+BTSPPCONN=0,0,\"%s\"\r\n",szDevice_Addr[0]);
		Uart.WriteString(SlaveAddrData);
	}else{
		printf("Not found device\n");
	}

	return bFindOK;

}


void LoopBack(CUart Uart,char *word)
{

	//TX
	int nlen = strlen(word);
	char senData[50];


	sprintf(senData,"AT+BTSPPSEND=0,%d\r\n",nlen);
	Uart.WriteString(senData);
	usleep(200000);
	SetupSend(Uart,word);

	usleep(1000000);
	//RX
	int  nReadLen;
    char szData[35];
	if(CommandRead(Uart, szData, sizeof(szData), &nReadLen)){
			szData[nReadLen] = 0;
		    printf("RX<--%s \r\n",szData);
	}
}

void Clean_Buffer(CUart Uart)
{
	char dump[100];
	int dumpLen;
	Uart.NoneBlockingRead((char *)&dump, sizeof(dump), &dumpLen);
}

int main()
{

	CUart Uart;
	Uart.Open(ESP32_UART_NAME);
 	char *szDeviceList[MaxList];
 	for(int i=0;i<MaxList;i++)
 		szDeviceList[i] = (char *)malloc(35*sizeof(char));

	printf("RFS2 SPP Demo (Master)!\n");

	printf("Reset Module\r\n");
	IOWR(ESP32_RST_BASE,0,0);
	usleep(500000);
	IOWR(ESP32_RST_BASE,0,1);
	usleep(2000000);
	SetupSend(Uart,"AT+SYSSTORE=0\r\n");

	//printf("Echo off\r\n");
	SetupSend(Uart,"ATE0\r\n");

	//printf("Classic Bluetooth Initialization\r\n");
	SetupSend(Uart,"AT+BTINIT=1\r\n");

	//printf("Set Module Name\r\n");
	SetupSend(Uart,"AT+BTNAME=\"ESP32_SPP_Master\"\r\n");

	//printf("Sets BT SCAN Mode\r\n");
	SetupSend(Uart,"AT+BTSCANMODE=2\r\n");

	//init classic bluetooth SPP profile, the role is master
	//printf("Set Master\r\n");
	SetupSend(Uart,"AT+BTSPPINIT=1\r\n");

	// search device list
	//GetDeviceList(Uart,"AT+BTSTARTDISC=0,5,10\r\n");

	//connect device (ESP32_SPP_Slave)
	Connect_Device(Uart,"ESP32_SPP_Slave",szDeviceList);
	usleep(2000000);

	Clean_Buffer(Uart);

	int count_test=0;
	char str[] = "[ ]LoopBack\n";

	while(1)
	{
		str[1] = '0' + count_test;
		count_test++;
		LoopBack(Uart,str);
		usleep(1000000);
		if(count_test==10)
			count_test=0;
	}

 	for(int i=0;i<MaxList;i++)
 		free(szDeviceList[i]);

  return 0;
}
