/*
 * "Hello World" example.
 *
 * This example prints 'Hello from Nios II' to the STDOUT stream. It runs on
 * the Nios II 'standard', 'full_featured', 'fast', and 'low_cost' example
 * designs. It runs with or without the MicroC/OS-II RTOS and requires a STDOUT
 * device in your system's hardware.
 * The memory footprint of this hosted application is ~69 kbytes by default
 * using the standard reference design.
 *
 * For a reduced footprint version of this template, and an explanation of how
 * to reduce the memory footprint for a given application, see the
 * "small_hello_world" template.
 *
 */

/*
 * "Hello World" example.
 *
 * This example prints 'Hello from Nios II' to the STDOUT stream. It runs on
 * the Nios II 'standard', 'full_featured', 'fast', and 'low_cost' example
 * designs. It runs with or without the MicroC/OS-II RTOS and requires a STDOUT
 * device in your system's hardware.
 * The memory footprint of this hosted application is ~69 kbytes by default
 * using the standard reference design.
 *
 * For a reduced footprint version of this template, and an explanation of how
 * to reduce the memory footprint for a given application, see the
 * "small_hello_world" template.
 *
 */


#include <stdio.h>
#include "CUart.h"
#include "terasic_includes.h"



bool CommandRead(CUart   &Uart, char *pBuffer, int nBufferSize, int *prx_lenmand){
	int RxLen;
	char Data;
	bool bDone = false;
	int Len = 0;
	alt_u32 timeout;

	if (!Uart.Read((char *)&Data, sizeof(Data), &RxLen) || RxLen == 0)
			return false;

	*(pBuffer + 0) = Data;

	// read until find '\r'
	//timeout = alt_nticks() + alt_ticks_per_second()/1000;
	timeout = alt_nticks() + (alt_ticks_per_second()*30);
	Len = 1;
	while(!bDone){
		if (Uart.NoneBlockingRead((char *)&Data, sizeof(Data), &RxLen) && RxLen == sizeof(Data)){
			*(pBuffer + Len) = Data;
			Len++;
			if (Data == '\n')
				bDone = true;
		}else if (Len >= nBufferSize){
			// buffer full
			bDone = true;
		}else if (alt_nticks() > timeout){
			// timeout
			bDone = true;
		}else{
		}
		usleep(10);

	}
	*prx_lenmand = Len;
	return true;
}

bool SetupSend(CUart Uart,char *cmd)
{
	bool bFindOK=false;
	int  nReadLen;
    char szData[100];
    alt_u32 timeout;
    printf("TX-->%s",cmd);

    timeout = alt_nticks() + (alt_ticks_per_second()*30);

	Uart.WriteString(cmd);
	while(!bFindOK){
	  if(CommandRead(Uart, szData, sizeof(szData), &nReadLen)){
	      szData[nReadLen] = 0;
	     // printf("RX<--%s \n",szData);
	  }
	  if(!(strcmp(szData, "OK\r\n")))
		  bFindOK = true;
	  if(alt_nticks() > timeout){
		  printf("Time Out\n");
		  return false;
	  }

	}
	usleep(100000);
	return bFindOK;
}


void SPP_TX(CUart Uart,char *cmd)
{

	int nlen = strlen(cmd);
	char senData[50];

	sprintf(senData,"AT+BTSPPSEND=0,%d\r\n",(char)nlen);
	Uart.WriteString(senData);
	usleep(200000);
	SetupSend(Uart,cmd);

}


void LoopBack(CUart Uart){

	int  nReadLen;
    char szData[35],szSendLen[35];
    char *pch;
    char Len[10];
    char szMessage[50];


	if(CommandRead(Uart, szData, sizeof(szData), &nReadLen)){
	    szData[nReadLen] = 0;

	    pch = strstr (szData,"+BTDATA:");
	    if(pch!=NULL){
	    	//RX
	    	printf("RX<--%s",szData);
	    	sscanf(szData,"+BTDATA:%[^,],%s",Len,szMessage);
	    	sprintf(szSendLen,"AT+BTSPPSEND=0,%s\r\n",Len);
	    	sprintf(szMessage,"%s\r\n",szMessage);
	    	//TX
	    	Uart.WriteString(szSendLen);
	    	usleep(100000);
	    	printf("TX:%s\r\n",szMessage);
	    	Uart.WriteString(szMessage);
	    	usleep(100000);
	    }
	}
}

int main()
{

	char SlaveAddrData[35];
	char inputData[35];
	CUart Uart;
	Uart.Open(ESP32_UART_NAME);


	printf("RFS2 SPP Demo (Slave)!\n");

	printf("Reset Module\r\n");
	IOWR(ESP32_RST_BASE,0,0);
	usleep(500000);
	IOWR(ESP32_RST_BASE,0,1);
	usleep(2000000);
	SetupSend(Uart,"AT+SYSSTORE=0\r\n");

	//printf("Echo off\r\n");
	SetupSend(Uart,"ATE0\r\n");

	//printf("Classic Bluetooth Initialization\r\n");
	SetupSend(Uart,"AT+BTINIT=1\r\n");

	//printf("Set Module Name\r\n");
	SetupSend(Uart,"AT+BTNAME=\"ESP32_SPP_Slave\"\r\n");

	//printf("Sets BT SCAN Mode\r\n");
	SetupSend(Uart,"AT+BTSCANMODE=2\r\n");

	//init classic bluetooth SPP profile, the role is slave
	//printf("Set Slave \r\n");
	SetupSend(Uart,"AT+BTSPPINIT=2\r\n");

	//Start the classic bluetooth SPP profile
	//printf("Start Bluetooth\r\n");
	SetupSend(Uart,"AT+BTSPPSTART\r\n");


	while(1){
		LoopBack(Uart);
	}



  return 0;
}
