
#include "ICM20948_AK09916.h"
#include <math.h>
#define G_SI 9.80665


static float gyro_scale_factor;
static float accel_scale_factor;
static float magnetometer_ASA[3];

int16_t gyroCount[3];   // Stores the 16-bit signed gyro sensor output
int16_t magCount[3];    // Stores the 16-bit signed magnetometer sensor output
int16_t accelCount[3];  // Stores the 16-bit signed accel sensor output

void ICM20948_I2C_Init(alt_u32 I2C_Controller_Base)
{
	const int ref_clk = 50*1000*1000; // 50Mhz
	const int i2c_clk = 400*1000; // 400KHz
	oc_i2c_init_ex(I2C_Controller_Base, ref_clk, i2c_clk);
}


bool ICM20948_Init()
{

	alt_u8 ICM20948_Read[1];
	bool bPass = true;

	// Reset ICM20948
  	writeByte(ICM20948_ADDRESS, PWR_MGMT_1,READ_FLAG);
  	delay(100);
	writeByte(ICM20948_ADDRESS, PWR_MGMT_1,0x01);
	delay(100);

	//check ICM20948 device id
	ReadBytes(ICM20948_ADDRESS,  WHO_AM_I_ICM20948,ICM20948_Read,1);
	delay(100);
	if(ICM20948_Read[0]!=0xea){
		printf("Device ID error\r\n");
		printf("ICM20948 Device ID must be 0xEA but read ID is 0x%x\r\n",ICM20948_Read[0]);
		bPass = false;
	}else{
		printf("ICM20948 ID: %x\r\n",ICM20948_Read[0]);
	}

	writeByte(ICM20948_ADDRESS, REG_BANK_SEL, 0x20);
	//align_enable
	writeByte(ICM20948_ADDRESS, ODR_ALIGN_EN,0x01);

	//Gyro_calibration();
	//Accel_calibration();
	Accel_full_scale_select(GFS_2000DPS);
	Gyro_full_scale_select(AFS_16G);
	Mag_calibration();

  	return bPass;

}
bool AK09916_Init()
{
	alt_u8 AK09916_Read[1];
	bool bPass = true;

	writeByte(ICM20948_ADDRESS, REG_BANK_SEL, 0x00);

	writeByte(ICM20948_ADDRESS, INT_PIN_CFG, 0x22);
	  // Enable data ready (bit 0) interrupt
	writeByte(ICM20948_ADDRESS, INT_ENABLE_1, 0x01);
	delay(100);

	ReadBytes(MAG_ADDRESS, WHO_AM_I_AK09916, AK09916_Read,1);
	delay(100);
	if(AK09916_Read[0]!=0x09){
		printf("Device ID error\r\n");
		printf("AK09916 Device ID must be 0x09 but read ID is 0x%x\r\n",AK09916_Read[0]);
		bPass = false;
	}else{
		printf("AK09916 ID: %x \r\n",AK09916_Read[0]);
	}

	//all registers are initialized
	writeByte(MAG_ADDRESS, AK09916_CNTL3, 0x01);
	delay(100);

	//
	writeByte(MAG_ADDRESS, AK09916_CNTL2, continuous_measurement_100hz );
	delay(100);


	return bPass;
}


unsigned int writeByte(uint8_t deviceAddress, uint8_t WriteAddr, uint8_t WriteData )
{
	OC_I2C_Write(MPU_I2C_OPENCORES_BASE, deviceAddress, WriteAddr, &WriteData, sizeof(WriteData));
	return 0x00;
}
uint8_t readByte(uint8_t deviceAddress,uint8_t registerAddress)
{
	unsigned char ReadData;
	OC_I2C_Read(MPU_I2C_OPENCORES_BASE, deviceAddress, registerAddress, &ReadData, sizeof(ReadData));
	return ReadData;
}

void ReadBytes(uint8_t deviceAddress, uint8_t ReadAddr, uint8_t *ReadBuf, unsigned int Bytes )
{
	OC_I2C_Read(MPU_I2C_OPENCORES_BASE, deviceAddress, ReadAddr, ReadBuf, Bytes);
}

void delay(alt_u32 time)
{
	usleep(time*10000);
}

void getMotion9(float *ax, float *ay, float *az, float *gx, float *gy, float *gz, float *mx, float *my, float *mz)
{

	// If intPin goes high, all data registers have new data
	// On interrupt, check if data ready interrupt
	if(readByte(ICM20948_ADDRESS,INT_STATUS_1) & 0X1){

		readAccelData(accelCount);
	    *ax = G_SI * (float)accelCount[0] /accel_scale_factor; // - accelBias[0];
	    *ay = G_SI * (float)accelCount[1] /accel_scale_factor; // - accelBias[1];
	    *az = G_SI * (float)accelCount[2] /accel_scale_factor; // - accelBias[2];


		readGyroData(gyroCount);
	    *gx = (M_PI/180) * (float)gyroCount[0]/gyro_scale_factor ; // - accelBias[0];
	    *gy = (M_PI/180) * (float)gyroCount[1]/gyro_scale_factor ; // - accelBias[1];
	    *gz = (M_PI/180) * (float)gyroCount[2]/gyro_scale_factor ; // - accelBias[2];


	    readMagData(magCount);
	    *mx = (float)magCount[0] * magnetometer_ASA[0];
	    *my = (float)magCount[1] * magnetometer_ASA[1];
	    *mz = (float)magCount[2] * magnetometer_ASA[2];

	}


}

void readAccelData(int16_t * destination)
{
  uint8_t rawData[6];  // x/y/z accel register data stored here
  // Read the six raw data registers into data array
  ReadBytes(ICM20948_ADDRESS,  ACCEL_XOUT_H, rawData, 6);
  // Turn the MSB and LSB into a signed 16-bit value
  destination[0] = (int16_t)(rawData[0] << 8) | rawData[1];
  destination[1] = (int16_t)(rawData[2] << 8) | rawData[3];
  destination[2] = (int16_t)(rawData[4] << 8) | rawData[5];
}

void readGyroData(int16_t * destination)
{
  uint8_t rawData[6];  // x/y/z gyro register data stored here
  // Read the six raw data registers sequentially into data array
  ReadBytes(ICM20948_ADDRESS,  GYRO_XOUT_H, rawData, 6);
  // Turn the MSB and LSB into a signed 16-bit value
  destination[0] = (int16_t)(rawData[0] << 8) | rawData[1];
  destination[1] = (int16_t)(rawData[2] << 8) | rawData[3];
  destination[2] = (int16_t)(rawData[4] << 8) | rawData[5];

}


void readMagData(int16_t * destination)
{
  // x/y/z gyro register data, ST2 register stored here, must read ST2 at end
  // of data acquisition
  uint8_t rawData[8];
  // Wait for magnetometer data ready bit to be set
  if (readByte(MAG_ADDRESS, AK09916_ST1) & 0x01)
  {
    // Read the six raw data and ST2 registers sequentially into data array
    ReadBytes(MAG_ADDRESS,  AK09916_XOUT_L, rawData, 8);
    uint8_t c = rawData[7]; // End data read by reading ST2 register
    // Check if magnetic sensor overflow set, if not then report data
	// Remove once finished

    if (!(c & 0x08))
    {
      // Turn the MSB and LSB into a signed 16-bit value
      destination[0] = ((int16_t)rawData[1] << 8) | rawData[0];
      // Data stored as little Endian
      destination[1] = ((int16_t)rawData[3] << 8) | rawData[2];
      destination[2] = ((int16_t)rawData[5] << 8) | rawData[4];
    }
  }
}

int16_t readTempData()
{
  uint8_t rawData[2]; // x/y/z gyro register data stored here
  // Read the two raw data registers sequentially into data array
  ReadBytes(ICM20948_ADDRESS,  TEMP_OUT_H, rawData, 2);
  // Turn the MSB and LSB into a 16-bit value
  return ((int16_t)rawData[0] << 8) | rawData[1];
}



void Gyro_calibration()
{

	int32_t gyro_bias[3] = {0};
	uint8_t gyro_offset[6] = {0};

	for(int i = 0; i < 100; i++)
	{
		readGyroData(gyroCount);  // Read the x/y/z adc values
		gyro_bias[0] += gyroCount[0];
		gyro_bias[1] += gyroCount[1];
		gyro_bias[2] += gyroCount[2];
	}

	gyro_bias[0] /= 100;
	gyro_bias[1] /= 100;
	gyro_bias[2] /= 100;

	// Construct the gyro biases for push to the hardware gyro bias registers,
	// which are reset to zero upon device startup.
	// Divide by 4 to get 32.9 LSB per deg/s to conform to expected bias input format.
	// Biases are additive, so change sign on calculated average gyro biases
	gyro_offset[0] = (-gyro_bias[0] / 4  >> 8) & 0xFF;
	gyro_offset[1] = (-gyro_bias[0] / 4)       & 0xFF;
	gyro_offset[2] = (-gyro_bias[1] / 4  >> 8) & 0xFF;
	gyro_offset[3] = (-gyro_bias[1] / 4)       & 0xFF;
	gyro_offset[4] = (-gyro_bias[2] / 4  >> 8) & 0xFF;
	gyro_offset[5] = (-gyro_bias[2] / 4)       & 0xFF;

	//delay(1000);
	//Bank_Switch(0x2);
	writeByte(ICM20948_ADDRESS, REG_BANK_SEL, 0x20);
	writeByte(ICM20948_ADDRESS, XG_OFFSET_H, gyro_offset[0]);
	writeByte(ICM20948_ADDRESS, XG_OFFSET_L, gyro_offset[1]);
	writeByte(ICM20948_ADDRESS, YG_OFFSET_H, gyro_offset[2]);
	writeByte(ICM20948_ADDRESS, YG_OFFSET_L, gyro_offset[3]);
	writeByte(ICM20948_ADDRESS, ZG_OFFSET_H, gyro_offset[4]);
	writeByte(ICM20948_ADDRESS, ZG_OFFSET_L, gyro_offset[5]);


}


void Accel_calibration()
{
	alt_u8 ICM20948_Read[2];
	int32_t accel_bias[3] = {0};
	int32_t accel_bias_reg[3] = {0};
	uint8_t accel_offset[6] = {0};

	for(int i = 0; i < 100; i++)
	{
		readAccelData(accelCount);
		accel_bias[0] += accelCount[0];
		accel_bias[1] += accelCount[1];
		accel_bias[2] += accelCount[2];
	}

	accel_bias[0] /= 100;
	accel_bias[1] /= 100;
	accel_bias[2] /= 100;

	uint8_t mask_bit[3] = {0, 0, 0};



	writeByte(ICM20948_ADDRESS, REG_BANK_SEL, 0x10);

	ReadBytes(ICM20948_ADDRESS,XA_OFFSET_H, ICM20948_Read, 2);
	accel_bias_reg[0] = (int32_t)(ICM20948_Read[0] << 8 | ICM20948_Read[1]);
	mask_bit[0] = ICM20948_Read[1] & 0x01;

	ReadBytes(ICM20948_ADDRESS,YA_OFFSET_H,ICM20948_Read, 2);
	accel_bias_reg[1] = (int32_t)(ICM20948_Read[0] << 8 | ICM20948_Read[1]);
	mask_bit[1] = ICM20948_Read[1] & 0x01;


	ReadBytes(ICM20948_ADDRESS,ZA_OFFSET_H, ICM20948_Read, 2);
	accel_bias_reg[2] = (int32_t)(ICM20948_Read[0] << 8 | ICM20948_Read[1]);
	mask_bit[2] = ICM20948_Read[1] & 0x01;

	accel_bias_reg[0] -= (accel_bias[0] / 8);
	accel_bias_reg[1] -= (accel_bias[1] / 8);
	accel_bias_reg[2] -= (accel_bias[2] / 8);

	accel_offset[0] = (accel_bias_reg[0] >> 8) & 0xFF;
  	accel_offset[1] = (accel_bias_reg[0])      & 0xFE;
	accel_offset[1] = accel_offset[1] | mask_bit[0];

	accel_offset[2] = (accel_bias_reg[1] >> 8) & 0xFF;
  	accel_offset[3] = (accel_bias_reg[1])      & 0xFE;
	accel_offset[3] = accel_offset[3] | mask_bit[1];

	accel_offset[4] = (accel_bias_reg[2] >> 8) & 0xFF;
	accel_offset[5] = (accel_bias_reg[2])      & 0xFE;
	accel_offset[5] = accel_offset[5] | mask_bit[2];

	//Bank_Switch;
	writeByte(ICM20948_ADDRESS, REG_BANK_SEL, 0x10);
	writeByte(ICM20948_ADDRESS, XA_OFFSET_H, accel_offset[0]);
	writeByte(ICM20948_ADDRESS, XA_OFFSET_L, accel_offset[1]);
	writeByte(ICM20948_ADDRESS, YA_OFFSET_H, accel_offset[2]);
	writeByte(ICM20948_ADDRESS, YA_OFFSET_L, accel_offset[3]);
	writeByte(ICM20948_ADDRESS, ZA_OFFSET_H, accel_offset[4]);
	writeByte(ICM20948_ADDRESS, ZA_OFFSET_L, accel_offset[5]);

}

void Mag_calibration()
{

	int i=0;
	float data;
	writeByte(ICM20948_ADDRESS, REG_BANK_SEL, 0x30);

	alt_u8 ICM20948_Read[3];
	ReadBytes(ICM20948_ADDRESS,  EXT_SENS_DATA_00,ICM20948_Read, 3);

	for(i=0;i<3;i++){
        data=ICM20948_Read[i];
        magnetometer_ASA[i]=((data-128)/256+1)*Magnetometer_Sensitivity_Scale_Factor;
	}
}


void Gyro_full_scale_select(Gyro_scale scale)
{
	alt_u8 ICM20948_Read[1];
	ReadBytes(ICM20948_ADDRESS,  GYRO_CONFIG_1,ICM20948_Read,1);
	switch(scale)
	{
		case GFS_250DPS :
			ICM20948_Read[0] |=  0x00;
			gyro_scale_factor = 131.0;
			break;
		case GFS_500DPS :
			ICM20948_Read[0] |=  0x02;
			gyro_scale_factor = 65.5;
			break;
		case GFS_1000DPS :
			ICM20948_Read[0] |=  0x04;
			gyro_scale_factor = 32.8;
			break;
		case GFS_2000DPS :
			ICM20948_Read[0] |=  0x06;
			gyro_scale_factor = 16.4;
			break;
	}
	writeByte(ICM20948_ADDRESS, REG_BANK_SEL, 0x20);
	writeByte(ICM20948_ADDRESS, GYRO_CONFIG_1, ICM20948_Read[0]);

}

void Accel_full_scale_select(Accel_Scale scale)
{

	alt_u8 ICM20948_Read[1];
	ReadBytes(ICM20948_ADDRESS, ACCEL_CONFIG, ICM20948_Read, 1);


	switch(scale)
	{
		case AFS_2G :
			ICM20948_Read[0] |=  0x00;
			accel_scale_factor = 16384;
			break;
		case AFS_4G :
			ICM20948_Read[0] |=  0x02;
			accel_scale_factor = 8192;
			break;
		case AFS_8G :
			ICM20948_Read[0] |=  0x04;
			accel_scale_factor = 4096;
			break;
		case AFS_16G :
			ICM20948_Read[0] |=  0x06;
			accel_scale_factor = 2048;
			break;
	}
	//switch bank
	writeByte(ICM20948_ADDRESS, REG_BANK_SEL, 0x20);

	//config accel
	writeByte(ICM20948_ADDRESS, ACCEL_CONFIG, ICM20948_Read[0]);

}

