/*
 * light_sensor.c
 *
 *  Created on: 201715
 *      Author: Administrator
 */


#include "terasic_includes.h"
#include "light_sensor.h"
#include "I2C_core.h"



bool I2C_Write_Light(alt_8 DeviceAddr, alt_u8 CommandCode, alt_u8 ControlData);
bool I2C_Write_Light16(alt_8 DeviceAddr, alt_u8 CommandCode, alt_u8 *pControlData);
bool I2C_Read_Light(alt_8 DeviceAddr, alt_u8 CommandCode, alt_u8 *pControlData);




////////////////////////////////////////
// export API
static alt_u32 Light_I2C_Controller_Base = LIGHT_I2C_OPENCORES_BASE;

void Light_Init(alt_u32 I2C_Controller_Base){
	const int Ref_CLK = 50*1000*1000; // 50MHz
	const int I2C_CLK = 400*1000; // 400KHz

	Light_I2C_Controller_Base = I2C_Controller_Base;
	oc_i2c_init_ex(Light_I2C_Controller_Base, Ref_CLK, I2C_CLK);
}


bool Light_Setup()
{
    bool bPass =true; //always true
    alt_u8 data = 0;

    I2C_Write_Light( DEVICE_ADDR, P_SYSM_CTRL 	, 0x03);
    I2C_Write_Light( DEVICE_ADDR, P_INT_CTRL 	, 0x03);
    I2C_Write_Light( DEVICE_ADDR, P_ALS_GAIN 	, 0x04);
    I2C_Write_Light( DEVICE_ADDR, P_ALS_TIME 	, 0x00);
    I2C_Write_Light( DEVICE_ADDR, P_LED_CTRL 	, 0xcf);
    I2C_Write_Light( DEVICE_ADDR, P_PS_GAIN 	, 0x03);
    I2C_Write_Light( DEVICE_ADDR, P_PS_PULSE 	, 0x08);
    I2C_Write_Light( DEVICE_ADDR, P_PS_TIME 	, 0x00);

    return bPass;
}

bool Light_Get_ADCData0(alt_u16 *pChannelData)
{
    bool bPass;
    alt_u8 Data0Low, Data0High;

    bPass = I2C_Read_Light( DEVICE_ADDR,P_CH0_DATA_L , &Data0Low);
    if(!bPass)
    {
         return FALSE;
    }
    else *pChannelData = Data0Low;

    if(bPass)
    {
        bPass = I2C_Read_Light( DEVICE_ADDR,P_CH0_DATA_H , &Data0High);
        if(!bPass)
        {
            return FALSE;
        }
        else *(pChannelData+1) = Data0High;
    }

    *pChannelData = (Data0High << 8) | Data0Low;

    return TRUE;
}

bool Light_Get_ADCData1(alt_u16 *pChannelData)
{
    bool bPass;
    alt_u8 Data1Low, Data1High;

    bPass = I2C_Read_Light( DEVICE_ADDR,P_CH1_DATA_L , &Data1Low);
    if(!bPass)
    {
         return FALSE;
    }
    else *pChannelData = Data1Low;

    if(bPass)
    {

        bPass = I2C_Read_Light(DEVICE_ADDR,P_CH1_DATA_H , &Data1High);
        if(!bPass)
        {
            return FALSE;
        }
        else *(pChannelData+1) = Data1High;
    }

    *pChannelData = (Data1High << 8) | Data1Low;

    return TRUE;
}

bool Get_light(alt_u16 *light0, alt_u16 *light1)
{
    bool bPass = FALSE;
    alt_u8 device_Id = 0;

    alt_u16 ChannelData0;
    alt_u16 ChannelData1;

    //get adc channel 0 data
    bPass = Light_Get_ADCData0(&ChannelData0);
    if(!bPass)
    {
        printf("Get ADC Channle 0 data fail \n");
        return FALSE;
    }
    *light0 = ChannelData0;

    //get adc channel 1 data
    bPass = Light_Get_ADCData1(&ChannelData1);
    if(!bPass)
    {
        printf("Get ADC Channle 1 data fail \n");
        return FALSE;
    }
    *light1 = ChannelData1;
    return TRUE;
}



/////////////////////////////////////////////
// internal
////////////////light sensor
bool I2C_Write_Light(alt_8 DeviceAddr, alt_u8 CommandCode, alt_u8 ControlData){
	bool bSuccess;

	bSuccess = OC_I2C_Write(Light_I2C_Controller_Base,DeviceAddr,CommandCode, &ControlData, 1);

    return bSuccess;


}

bool I2C_Write_Light16(alt_8 DeviceAddr, alt_u8 CommandCode, alt_u8 *pControlData){
	bool bSuccess;

	bSuccess = OC_I2C_Write(Light_I2C_Controller_Base,DeviceAddr,CommandCode, pControlData, 2);

    return bSuccess;


}

bool I2C_Read_Light(alt_8 DeviceAddr, alt_u8 CommandCode, alt_u8 *pControlData){
    bool bSuccess;

    bSuccess = OC_I2C_Read(Light_I2C_Controller_Base,DeviceAddr,CommandCode, pControlData, 1);

    return bSuccess;
}



