module UART_ST  (
  input             RX  , 
  input  [159:0]    NAME , 
  input  [159:0]    PASSWORD, 
  input  [7:0]      NAME_LN    ,
  input  [7:0]      PASSWORD_LN,
 
  input             CLK,
  input             RESET_N  ,
  output reg        ADDRESS ,
  output reg        CS ,
  output reg [3:0]  BYTE_ENABLE ,
  output reg        READ,
  output reg        WRITE,
  output reg [31:0] WRITEDATA ,
  output reg [63:0] DELAY , 
  input   [31:0]    READDATA,
  output reg        READY  , 
  
  //
  output reg [7:0]  ST 	,
  output     [7:0]  WRCM1,
  output     [7:0]  WRCM2,
  output     [7:0]  WRCM3,
  output     [7:0]  WRCM4,
  output     [7:0]  WRCM5,  
  output     [7:0]  WRCM6,
  output     [7:0]  WRCM7,

  output reg [7:0]  CNT ,
  output     [7:0]  ASCII_TX,
  output     [7:0]  ASCII_RX,
  output     [7:0]  RD_BUFF_CNT,
  
  output reg [7:0]    RD_LENTH, 
  output reg [7:0]    MESSAGE,
  output reg [7:0]    WR_LEN ,
  output              WI ,RI,
  output     [7:0]    Second ,
  output     [7:0]    Minute ,  
  output     [7:0]    Hour   ,
  output reg [15:0]   M_ADDR , 
  output reg          M_WR  
  
  
  
);
 wire [7:0]    SS;
 wire [7:0]    MM;  
 wire [7:0]    HH;
 assign        Second= SS;
 assign        Minute= MM;
 assign        Hour  = HH;
 


parameter  RE=1'b1; //1 :RE R/W Interrupt-enable bit for read interrupts  BIT0 
parameter  WE=1'b1; //1 :WE R/W Interrupt-enable bit for write interrupts BIT1

parameter  TIME_BASE     = 50000000 ;
parameter  S05= TIME_BASE  /2; 
parameter  S1= TIME_BASE  * 1; 
parameter  S2= TIME_BASE  * 2; 
parameter  S10= TIME_BASE * 10;

parameter  MASSEGE_LN_WR1 =15;
parameter  MASSEGE_LN_WR2 =13;
parameter  MASSEGE_LN_WR4 =41;
parameter  MASSEGE_LN_WR5 =15;
parameter  MASSEGE_LN_WR6 =72;

parameter  MASSEGE_NUM   =6;
parameter  STEP7_MESS_LN =1024 ;  

//=======================================================
//  REG/WIRE declarations
//======================================================= 

wire [7:0]   MASSEGE_LN_WR3; 

//=======================================================
//  Structural coding
//=======================================================

//--STEP 3 INPUT NAME/PASSWORD L -- 
assign MASSEGE_LN_WR3  = 15 + NAME_LN + PASSWORD_LN +1 ; 

//--BUS -- 
 assign  ASCII_TX    = WRITEDATA[7:0] ; 
 assign  ASCII_RX    = READDATA[7:0] ; 
 assign  RD_BUFF_CNT = READDATA[23:16] ; 
 assign  {WI,RI }    = ADDRESS ? READDATA[9:8]  : 2'h0 ; 

//---WIFI TIME READ HH:MM:SS  --
RX_TIME_ASCII_BCD  md(
   .CK_50  (CLK), 
   .RST_N  (READY), 
   .RX     (RX),
   .SS     (SS ),
   .MM     (MM ),
   .HH     (HH )
);
 

//------------st ------
always @(negedge RESET_N or posedge  CLK ) 
if ( !RESET_N) begin
  READY      <=0; 
  ST         <= 0; 
  ADDRESS    <=0;  
  BYTE_ENABLE<=4'hf ;  
  CS         <=0;  
  WRITE      <=0;  
  CNT        <=0;  
  WRITEDATA  <=32'h0;  
  DELAY      <=0; 
  MESSAGE    <=1; 
  M_ADDR     <=0;  
  M_WR       <=0;  
  
end
else begin 
  case (ST) 
  0:begin  
           if  (DELAY != S2 )  begin 
			      DELAY <=DELAY+1 ; 
				   CS          <= ~CS   ;  
				   READ        <= ~READ ;  
               M_ADDR    <= M_ADDR +1; 					
               M_WR      <=1; 
 			   end 
			   else  begin 
				  M_ADDR    <=0; 
				  M_WR      <=0; 	
				  ST        <=1;  
              ADDRESS   <=1 ;  
              WRITEDATA <={WE , RE }  ;  //SET WRITE READ BUFFER INTERRUPT      				
				  READY     <=1; 
				  MESSAGE   <=1; 
				  CS        <=1;
			 end 
	end  
 
 1:begin   WRITE <=1; ST<= 2;  end 
 2:begin   WRITE <=0; ST<= 3;  end 
 
 //----- WRITE TO ESP32-WROOM-32E COMMAND 
 3:begin
 				  CS     <= 0;
              ADDRESS<= 0; 			  
                   if ( MESSAGE ==1 ) begin WR_LEN<= MASSEGE_LN_WR1; end
				  else if ( MESSAGE ==2 ) begin WR_LEN<= MASSEGE_LN_WR2; end
				  else if ( MESSAGE ==3 ) begin WR_LEN<= MASSEGE_LN_WR3; end
				  else if ( MESSAGE ==4 ) begin WR_LEN<= MASSEGE_LN_WR4; end
				  else if ( MESSAGE ==5 ) begin WR_LEN<= MASSEGE_LN_WR5; end
				  else if ( MESSAGE ==6 ) begin WR_LEN<= MASSEGE_LN_WR6; end

				  if ( ( MESSAGE ==5 ) &&  ( DELAY != S1 ) )  DELAY <=DELAY +1 ; 
				  else 
				    ST  <=4;  
				    CNT <=0 ; 
				  end  
 
 4:begin  ST  <=5;   end  
				 
 5:begin 
          ST <=6;  CS <=1; CNT <=CNT+1 ; 
          if ( MESSAGE ==1 ) begin  WRITEDATA <= WRCM1[7:0] ;   end			
	  else if ( MESSAGE ==2 ) begin  WRITEDATA <= WRCM2[7:0] ;   end			
	  else if ( MESSAGE ==3 ) begin  WRITEDATA <= WRCM3[7:0] ;   end			
	  else if ( MESSAGE ==4 ) begin  WRITEDATA <= WRCM4[7:0] ;   end			
	  else if ( MESSAGE ==5 ) begin  WRITEDATA <= WRCM5[7:0] ;   end			
	  else if ( MESSAGE ==6 ) begin  WRITEDATA <= WRCM6[7:0] ;   end							
	end 
	
6:begin ST <=7; WRITE <=1;  end 
7:begin ST <=8; WRITE <=0; end  	
	
8:begin  
      CS <=0; 
		            M_ADDR <= M_ADDR+1; 
      if (   CNT != WR_LEN) begin 
	          ST<=5; 
		end 
	   else begin 
			  CNT    <=  0 ;
           ST     <=  9;
           DELAY  <=  0 ; 
			  M_WR <= 0 ; 
		end 
   end  //-----END WRITE  ------- 
	
//---READ----
9:begin  
		     if (  
				  (( MESSAGE ==1 ) && (DELAY !=S05  )) ||  
			     (( MESSAGE ==2 ) && (DELAY !=S2   )) ||  
			     (( MESSAGE ==3 ) && (DELAY !=S10  )) ||   
			     (( MESSAGE ==4 ) && (DELAY !=S05  )) ||  
				  (( MESSAGE ==5 ) && (DELAY !=S05  )) ||  
			     (( MESSAGE ==6 ) && (DELAY !=S05 ))      
				  )
           begin
					 DELAY <=DELAY+1 ; 
           end					 
           else begin 
                ST     <=10 ; 	
		          M_WR   <=1  ; 			 
           end 
 	end 
10:begin 
    CS   <=0; 
	 if ( MESSAGE == MASSEGE_NUM )  ST<= 11;
	 else begin  
	         MESSAGE<= MESSAGE+1 ;  
				ST     <= 3; 
	         DELAY  <= 0; 
				CNT    <= 0 ;
	 end 
end //---read end ---- //

11:begin  
	 if (  { SS , MM, HH } ==0  ) begin
              CS         <=0;  
              WRITE      <=0;  
              CNT        <=0;  
              WRITEDATA  <=32'h0;  
              DELAY      <=0;
				  ST         <=0; 
				  READY      <=0; 
	 end
	 else   begin 
	    	 ST      <=12; 
			 DELAY   <=0; 
	 end 	 
	 end 
//-----ST-END 	
12:begin  
            ST     <= 3;//12 
	         MESSAGE<= 4 ;  
	         DELAY  <= 0; 
				CNT    <= 0 ;
   end
 endcase   
end




//====== TX :SEND TO ESP32-WROOM-32E MESSAGE DATA  TABEL ======
//-----STEP1
 WRITE_STEP1  stp1( 
     .ADDRESS  ( CNT  ), 
     .ROM_DATA ( WRCM1)
 );
//-----STEP2 
 WRITE_STEP2  stp2( 
     .ADDRESS  ( CNT), 
     .ROM_DATA ( WRCM2)
 );
 
//-----STEP3   //password 
 WRITE_STEP3  stp3( 
     .ADDRESS  ( CNT), 
	  .CLK      ( CLK ) ,
	  .RESET_N  ( RESET_N &  ~( MESSAGE!=3 )), 
     .ROM_DATA ( WRCM3),
	  .N        ( NAME ) ,
	  .P        ( PASSWORD ),
	  .L1       ( NAME_LN    ) ,
	  .L2       ( PASSWORD_LN) ,
	  
 );
 
//-----STEP4
 WRITE_STEP4  stp4( 
     .ADDRESS  ( CNT), 
     .ROM_DATA ( WRCM4)
 );

 
//-----STEP5
 WRITE_STEP5  stp5( 
     .ADDRESS  ( CNT), 
     .ROM_DATA ( WRCM5)
 ); 
 
//-----STEP6
 WRITE_STEP6  stp6( 
     .ADDRESS  ( CNT), 
     .ROM_DATA ( WRCM6)
 ); 


endmodule 
