
module  APM_16D24 (  // LIGHT Digital Sensor // 
   input            RESET_N ,
	input            CLK_50, 
	output           I2C_SCL, 
	inout            I2C_SDA, 
	
   //Ambient Light Sensor DATA
	output reg [7:0] CH0_DATA_L,//ALS DATA
	output reg [7:0] CH0_DATA_H,//ALS DATA
	output reg [7:0] CH1_DATA_L,//ALS DATA
	output reg [7:0] CH1_DATA_H,//ALS DATA
	
	//Proximity Sensor DATA
   output reg [7:0] PS_DATA_L ,//PS DATA	
   output reg [7:0] PS_DATA_H ,//PS DATA	
	
	//IR Sensor DATA
   output reg [7:0] IR_DATA_L ,//IR DATA	
   output reg [7:0] IR_DATA_H, //IR DATA	
	
	//Product number
	output reg [7:0] PNO_LB,
	output reg [7:0] PNO_HB,
	
	//POR
	output reg [7:0] INT_FLAG,
	
	//---16bit data
	output     [15:0]CH0_DATA,//ALS channel 0
	output     [15:0]CH1_DATA,//ALS channel 1	
	output     [15:0]PS_DATA,
	output     [15:0]IR_DATA,
	output     [15:0]PNO ,    //default 16'h1614 
	
	
	//---FOR TEST 
	output           CLK_400K ,
   output reg       I2C_LO0P,
   output reg [7:0] ST ,
   output reg [7:0] CNT,
	output           W_WORD_END ,
   output reg       W_WORD_GO ,	
	output [7:0]     WORD_ST,
	output [7:0]     WORD_CNT,
	output [7:0]     WORD_BYTE	,
   output [7:0]     R_DATA,
   output reg    [7:0] W_POINTER_REG
		
	);
//--APM16D24 --REGISTER	
`include "APM_16D24.h"

//-Pointer Register Number-- 
parameter READ_REG_MAX      =11;
parameter WRITE_REG_MAX     =8;


//-- ALS Data
	assign CH0_DATA ={CH0_DATA_H ,CH0_DATA_L} ;
	assign CH1_DATA ={CH1_DATA_H ,CH1_DATA_L} ;	
	
//-- PS Data
	assign PS_DATA ={PS_DATA_H ,PS_DATA_L} ;

//-- ALS Data
	assign IR_DATA ={IR_DATA_H ,IR_DATA_L} ;
	
//-- Product number	
   assign PNO 	  = { PNO_HB , PNO_LB };//default 16'h1614 
	
	
//-- I2C clock 100k generater 
CLOCKMEM MM(  .CLK(CLK_50) ,.CLK_FREQ(500), .CK_1HZ (CLK_400K) ) ;
  
 
//--ST--
reg [7:0] DELY ; //ST DELAY
always @(negedge RESET_N or posedge CLK_400K )
begin 
 if (!RESET_N)   begin 
     ST           <=0;	  
	  W_POINTER_GO <=1;   
	  R_GO         <=1;
	  W_WORD_GO    <=1;
  end 	  
 else 
case (ST)
0: begin 
    ST           <=30;
	 W_POINTER_GO <=1;   
	 R_GO         <=1;
	 W_WORD_GO    <=1;
  end
//<----------------READ -------	
1: begin 
     ST<=2; 
	end	
2: begin 

        if ( CNT==0 )  W_POINTER_REG <=P_INT_FLAG   ; 
   else if ( CNT==1 )  W_POINTER_REG <=P_PNO_LB     ; 
   else if ( CNT==2 )  W_POINTER_REG <=P_PNO_HB     ; 
	else if ( CNT==3 )  W_POINTER_REG <=P_CH0_DATA_L ; 
   else if ( CNT==4 )  W_POINTER_REG <=P_CH0_DATA_H ;
	else if ( CNT==5 )  W_POINTER_REG <=P_CH1_DATA_L ;  
	else if ( CNT==6 )  W_POINTER_REG <=P_CH1_DATA_H ;  
	else if ( CNT==7 )  W_POINTER_REG <=P_PS_DATA_L  ; 
	else if ( CNT==8 )  W_POINTER_REG <=P_PS_DATA_H  ; 
	else if ( CNT==9 )  W_POINTER_REG <=P_IR_DATA_L  ; 
	else if ( CNT==10)  W_POINTER_REG <=P_IR_DATA_H  ; 	
	
	if ( W_POINTER_END ) 
	  begin  
	    W_POINTER_GO  <=0; 
		 ST<=3 ; 
		 DELY<=0;  
		end // Write ID pointer 	 
	end                 
3: begin 
   DELY  <=DELY +1;
   if ( DELY ==2 ) begin 
     W_POINTER_GO  <=1;
     ST<=4 ; 
	end
	end       
4: begin 
   if  ( W_POINTER_END ) ST<=5 ; 	
	end              
5: begin ST<=6 ; end //delay
	
//read DATA 		 
6: begin 
	if ( R_END ) begin  R_GO  <=0; ST<=7 ; DELY<=0; end
	end  
	             
	// Write ID pointer 
7: begin 
    DELY  <=DELY +1;
    if ( DELY ==2 ) 
	 begin 
      R_GO  <=1;
      ST<=8 ; 
	 end
	end       
8: begin 
    ST<=9 ; 
	end 	
9: begin 
    if  ( R_END ) 
	 begin 
	       if ( CNT==0 ) INT_FLAG   <=R_DATA ; 
	  else if ( CNT==1 ) PNO_LB     <=R_DATA ; 
	  else if ( CNT==2 ) PNO_HB     <=R_DATA ; 
	  else if ( CNT==3 ) CH0_DATA_L <=R_DATA ; 
	  else if ( CNT==4 ) CH0_DATA_H <=R_DATA ; 			 
	  else if ( CNT==5 ) CH1_DATA_L <=R_DATA ; 
	  else if ( CNT==6 ) CH1_DATA_H <=R_DATA ;  
	  else if ( CNT==7 ) PS_DATA_L  <=R_DATA ; 
	  else if ( CNT==8 ) PS_DATA_H  <=R_DATA ; 			 
	  else if ( CNT==9 ) IR_DATA_L  <=R_DATA ; 
	  else if ( CNT==10) IR_DATA_H  <=R_DATA ; 	  
  
	  ST  <=10 ; 	
	  CNT<=CNT+1 ;
	 end
	end              
10: begin   
      if ( ( INT_FLAG & 8'h80 )   !=8'h80  ) ST<= 30 ;
		else  begin 
	   if (CNT == READ_REG_MAX)       begin CNT <= 0 ; end 
	    W_POINTER_GO <=1;
       R_GO         <=1 ;		 
	    W_WORD_GO    <=1;
		 ST<=1 ; 
		end  
		  
	 end	
//<----------------------------------WRITE WORD-----------------
30: begin 
    ST<=31; 
	 CNT=0 ; 
    end	
31: begin 
			        if (CNT==0 )    {W_WORD_REG[7:0] ,W_WORD_DATA[7:0]} <= { P_SYSM_CTRL[7:0]  ,8'h03 };//Enable PS function ,Enables ALS function
			  else  if (CNT==1 )    {W_WORD_REG[7:0] ,W_WORD_DATA[7:0]} <= { P_INT_CTRL [7:0]  ,8'h03 };//Enable INT_PS effect INT pin  , Enable INT_ALS effect INT pin				
			  else  if (CNT==2 )    {W_WORD_REG[7:0] ,W_WORD_DATA[7:0]} <= { P_ALS_GAIN [7:0]  ,8'h04 };//x96
			  else  if (CNT==3 )    {W_WORD_REG[7:0] ,W_WORD_DATA[7:0]} <= { P_ALS_TIME [7:0]  ,8'h00 };//default 
			  else  if (CNT==4 )    {W_WORD_REG[7:0] ,W_WORD_DATA[7:0]} <= { P_LED_CTRL [7:0]  ,8'hcf };//bit[7:6] =3,200 mA  .bit[5:0] =3f, 15T(15x13.675 us)
			  else  if (CNT==5 )    {W_WORD_REG[7:0] ,W_WORD_DATA[7:0]} <= { P_PS_GAIN  [7:0]  ,8'h03 };//x8 
			  else  if (CNT==6 )    {W_WORD_REG[7:0] ,W_WORD_DATA[7:0]} <= { P_PS_PULSE [7:0]  ,8'h08 };//15 pulses 
			  else  if (CNT==7 )    {W_WORD_REG[7:0] ,W_WORD_DATA[7:0]} <= { P_PS_TIME  [7:0]  ,8'h00 };//default 
			 
		
	   if (  W_WORD_END ) 
	   begin  
	     W_WORD_GO  <=0; 
		  ST<=32 ; 
		  DELY<=0; 
	   end
	end     
32: begin 
      DELY  <=DELY +1;
      if ( DELY ==2 ) 
		  begin 
          W_WORD_GO  <=1;
          ST<=33 ; 
	     end
	 end       
33: begin 
    ST<=34 ; 
	end       	
34: begin 
     if  ( W_WORD_END )  begin 
			ST<=35 ; 
		   CNT <=CNT +1 ;	
	  end
	end              
35: begin 
	 if   ( CNT== WRITE_REG_MAX  )  begin ST <=1 ;  CNT<=0 ; end 
	 else ST <= 31 ; 
	 end 
endcase 
end 

//--I2C-BUS-WIRE--
wire SDAO;
assign I2C_SCL = W_POINTER_SCL  & R_SCL  & W_WORD_SCL; 
assign SDAO    = W_POINTER_SDAO & R_SDAO & W_WORD_SDAO ;
assign I2C_SDA = (SDAO)?1'bz :1'b0 ; 


//----I2C WRITE WORD--
wire          W_WORD_SCL ; 
reg    [7:0]  W_WORD_REG ; 
reg    [15:0] W_WORD_DATA;
wire          W_WORD_SDAO ;  

I2C_WRITE_WORD_2BYTE  wrd(
   .RESET_N      ( RESET_N),
	.PT_CK        ( CLK_400K),
	.GO           ( W_WORD_GO),
	.POINTER      ( W_WORD_REG),
   .WDATA0	     ( W_WORD_DATA),		
	//WDATA1,		
	.NUM  ( 1),   //write 1byte
	.SLAVE_ADDRESS( APM16D24_ADDRESS),
	.SDAI         ( I2C_SDA),
	.SDAO         ( W_WORD_SDAO),
	.SCLO         ( W_WORD_SCL ),
	.END_OK       ( W_WORD_END),
	//--for test 
	.ST           ( WORD_ST ),
	.CNT          ( WORD_CNT),
	.BYTE         ( WORD_BYTE),
	.ACK_OK       ( )
);

//----I2C WRITE POINTER---
wire         W_POINTER_SCL ; 
wire         W_POINTER_END ; 
reg          W_POINTER_GO ; 

wire         W_POINTER_SDAO ;  
I2C_WRITE_POINTER  wpt(
   .RESET_N      ( RESET_N),
	.PT_CK        ( CLK_400K),
	.GO           ( W_POINTER_GO),
	.POINTER      ( W_POINTER_REG),
	.SLAVE_ADDRESS(APM16D24_ADDRESS),	
	.SDAI         ( I2C_SDA),
	.SDAO         ( W_POINTER_SDAO),
	.SCLO         ( W_POINTER_SCL ),
	.END_OK       ( W_POINTER_END),
	//--for test 
	.ST           (),
	.ACK_OK       (),
	.CNT          (),
	.BYTE         ()  	
);


//------I2C READ DATA --- 
wire R_SCL; 
wire R_END; 
reg  R_GO; 
wire R_SDAO; 
I2C_READ_2BYTE  rid( 	
   .RESET_N      (RESET_N),
	.PT_CK        (CLK_400K),
	.GO           (R_GO),
	.SLAVE_ADDRESS(APM16D24_ADDRESS),	
	.SDAI         (I2C_SDA),
	.SDAO         (R_SDAO),
	.SCLO         (R_SCL),
	.END_OK       (R_END),
	.DATA16       (R_DATA[7:0]),
	.END_BYTE     (0) 

);

endmodule 
	
	
	