/*
 * esp32.c
 *
 *  Created on: 2016/10/7
 *      Author: User
 */

#include <stdio.h>
#include <stdlib.h>
#include <stdbool.h>
#include "system.h"
#include <fcntl.h>
#include <unistd.h>
#include "esp32.h"
#include <string.h>
#include <altera_avalon_pio_regs.h>

#define esp32_uart WIFI_UART_NAME
//#define ESP32_DEBUG

FILE *esp32_file;

void set_esp32_file_blocking(bool blocking)
{
    if (blocking == false) {
        fcntl(fileno(esp32_file), F_SETFL, O_NONBLOCK);
    } else {
        int file_fl = fcntl(fileno(esp32_file), F_GETFL);
        file_fl = file_fl & ~O_NONBLOCK;
        fcntl(fileno(esp32_file), F_SETFL, file_fl);
    }
}

// ESP32-WROOM-32
// https://docs.espressif.com/projects/esp-at/en/latest/AT_Command_Set/Wi-Fi_AT_Commands.html
bool esp32_init(bool reset)
{
    bool bSuccess = true;

    esp32_file = fopen(esp32_uart, "rw+");
    if (esp32_file == NULL) {
        printf("Open UART failed\n");
        return false;
    }

    // enable
    if (reset) {
        IOWR_ALTERA_AVALON_PIO_DATA(PIO_WIFI_ENABLE_BASE, 0); // power off the chip
        usleep(20*1000);
        IOWR_ALTERA_AVALON_PIO_DATA(PIO_WIFI_ENABLE_BASE, 1); // power on the chip
        usleep(3 * 1000 * 1000); // wait ready
        esp32_dump_rx();
    }

    char ssid[32], passwd[32];

#ifdef ESP32_DEBUG
    esp32_send_command("AT");
    //esp32_send_command("AT+CMD?"); //List all AT commands and types supported in current firmware ... not work!
    //esp32_send_command("AT+RFPOWER?"); //Query the RF TX Power
    esp32_send_command("AT+GMR"); //Check Version Information
#endif

    esp32_send_command("AT+SYSSTORE=0"); //not store configuration into flash

    esp32_send_command("AT+UART_CUR=115200,8,1,0,3"); //enable both RTS and CTS
    //esp32_send_command("AT+UART_CUR?");



//    esp32_send_command("AT+CWMODE_CUR=1");
    esp32_send_command("AT+CWMODE=1"); // set WiFI RF to Station mode


//    esp32_send_command("AT+CWLAPOPT=1,0x2");
    esp32_send_command("AT+CWLAPOPT=1,6"); //Set the configuration for the command AT+CWLAP. ssid+rssi will be shown


    printf("Network Name (SSID) List:\n");
    esp32_send_command("AT+CWLAP");

    printf("Enter the Network Name (SSID): ");
    scanf("%s", ssid);
    printf("\n");
    printf("Enter the Password of Network Name (SSID): ");
    scanf("%s", passwd);
    printf("\n");


    printf("Connecting to WiFi AP (SSID: %s)\n", ssid);
    char cmd[100];


//    sprintf(cmd, "AT+CWJAP_CUR=\"%s\",\"%s\"", ssid, passwd);
    sprintf(cmd, "AT+CWJAP=\"%s\",\"%s\"", ssid, passwd);
    bSuccess = esp32_send_command(cmd);
    if (bSuccess) {
        printf("Connect to WiFi AP successfully\n");
    } else {
        printf("Connect to WiFi AP failed\n");
    }

    return bSuccess;
}

char *esp32_gets(char *buffer, int buffer_size)
{
    return fgets(buffer, buffer_size, esp32_file);
}

bool esp32_send_command(const char *cmd)
{
    fprintf(esp32_file, "%s\r\n", cmd);
    int length = 0;
    static char buffer[1000];

    usleep(20*1000);
    while (1) {
        if (fgets(buffer + length, sizeof(buffer) - length,
                esp32_file) != NULL) {
#ifdef ESP32_DEBUG
            printf("%s", buffer + length);
#endif
            if (strstr(buffer + length, "OK") != NULL) {
                if (strcmp("AT+CWLAP", cmd) == 0) {
                    printf("%s", buffer);
                }
                usleep(20*1000);
                return true;
            } else if (strstr(buffer + length, "ERROR") != NULL) {
                return false;
            } else if (strstr(buffer + length, "FAIL") != NULL) {
                return false;
            }
            length += strlen(buffer + length);
        }
    }
    return false;
}

bool esp32_send_data(const char *data, int length)
{
    write(fileno(esp32_file), data, length);

    length = 0;
    char buffer[1000];
    while (1) {
        if (fgets(buffer + length, sizeof(buffer) - length,
                esp32_file) != NULL) {
#ifdef ESP32_DEBUG
            printf("%s", buffer + length);
#endif
            if (strstr(buffer + length, "SEND OK") != NULL) {
                return true;
            } else if (strstr(buffer + length, "SEND FAIL") != NULL) {
                printf("%s", buffer);
                return false;
            }
            length += strlen(buffer + length);
        }
    }
    return false;
}

void esp32_dump_rx()
{
    char buffer[1000];
    set_esp32_file_blocking(false);
    while (esp32_gets(buffer, sizeof(buffer)) != NULL) {
#ifdef ESP32_DEBUG
        printf("%s", buffer);
#endif
    }
    set_esp32_file_blocking(true);
    fflush(stdout);
}
